# :coding: utf-8
# :copyright: Copyright (c) 2021 Lluis Casals Marsol
# Please see the LICENSE file that should have been included as part of this
# package.

#Available chains id for historical prices, probably for all the queries too.
AVAILABLE_CHAIN_IDS = {
    'Ethereum Mainnet': '1',
    'Polygon/Matic Mainnet': '137',
    'Polygon/Matic Mumbai Testnet': '80001',
    'Binance Smart Chain': '56',
    'Avalanche C-Chain Mainnet': '43114',
    'Fuji C-Chain Testnet': '43113',
    'Fantom Opera Mainnet': '250'
}

#Available chains id contract metadata
AVAILABLE_CHAIN_IDS_CONTRACT_METADATA = {
    'Polygon/Matic Mainnet': '137',
    'Binance Smart Chain': '56',
    'Avalanche C-Chain Mainnet': '43114',
    'Fuji C-Chain Testnet': '43113',
    'Fantom Opera Mainnet': '250'
}